-- Table
GameState = {}

-- New
function GameState:new()
	local gs = {}

	gs = setmetatable(gs, self)
	self.__index = self
	_gs = gs
	
	return gs
end

-- Load
function GameState:load()
	love.graphics.setDefaultFilter("nearest", "nearest")

	playerImg = love.graphics.newImage("assets/player/base.png")
	local grid = anim8.newGrid(24, 24, playerImg:getWidth(), playerImg:getHeight())
	playerAnim = {
		idle = anim8.newAnimation(grid('1-2', 1), 0.5),
		running = anim8.newAnimation(grid('3-6', 1), 0.2),
	}

	enemyImg = love.graphics.newImage("assets/npc/zombie.png")
	local grid = anim8.newGrid(24, 24, enemyImg:getWidth(), enemyImg:getHeight())
	enemyAnim = {
		Standard = {
			img = love.graphics.newImage("assets/npc/zombie.png"),
			idle = anim8.newAnimation(grid('1-2', 1), 0.5),
			running = anim8.newAnimation(grid('3-6', 1), 0.2),
		},
		Stricker = {
			img = love.graphics.newImage("assets/npc/stricker.png"),
			idle = anim8.newAnimation(grid('1-2', 1), 0.5),
			running = anim8.newAnimation(grid('3-6', 1), 0.2),
		},
		Heavy = {
			img = love.graphics.newImage("assets/npc/heavy.png"),
			idle = anim8.newAnimation(grid('1-2', 1), 0.5),
			running = anim8.newAnimation(grid('3-6', 1), 0.2),
		},
		Physic = {
			img = love.graphics.newImage("assets/npc/physic.png"),
			idle = anim8.newAnimation(grid('1-2', 1), 0.5),
			running = anim8.newAnimation(grid('3-6', 1), 0.2),
		},
		Robot = {
			img = love.graphics.newImage("assets/npc/robot.png"),
			idle = anim8.newAnimation(grid('1-2', 1), 0.5),
			running = anim8.newAnimation(grid('3-6', 1), 0.2),
		},
	}

	ak47 = love.graphics.newImage("assets/guns/ak47.png")
	sniper = love.graphics.newImage("assets/guns/sniper.png")
	barrel = love.graphics.newImage("assets/guns/barrel.png")
	bazooka = love.graphics.newImage("assets/guns/bazooka.png")
	tazer = love.graphics.newImage("assets/guns/tazer.png")

	bBullet = love.graphics.newImage("assets/bullets/bullet_ussrheavy.png")
	bullet = love.graphics.newImage("assets/bullets/bullet.png")
	missile = love.graphics.newImage("assets/bullets/missile.png")
	energy = love.graphics.newImage("assets/bullets/energy.png")
	
	health = love.graphics.newImage("assets/health.png")
	muzzle = love.graphics.newImage("assets/guns/muzzle.png")
	explosion = love.graphics.newImage("assets/explosion.png")

	newSound("assets/sound/gun.ogg")

	width, height = love.graphics.getWidth(), love.graphics.getHeight()
	
	font16 = love.graphics.newFont("assets/Simonetta-Regular.ttf", 16)
	font42 = love.graphics.newFont("assets/Simonetta-Regular.ttf", 42)
	
	tiles = {}
	tiles[0] = love.graphics.newImage("assets/tiles/floor.png")
	tiles[1] = love.graphics.newImage("assets/tiles/fence.png")
	tiles[2] = love.graphics.newImage("assets/tiles/manHole.png")
	
	map = {
		{ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 },
	}
	
	mapp = {}
	mapp.tileWidth = 32
	mapp.tileHeight = 32

	addGun("AK-47", ak47, bullet, 0.08, 900, 2)
	addGun("Sniper", sniper, bullet, 0.8, 1000, 1000)
	addGun("Barrel", barrel, bBullet, 0.03, 1000, 1)
	addGun("Bazooka", bazooka, missile, 2, 800, 1200)
	addGun("Tazer", tazer, energy, 0.5, 1000, 0)

	equipGun("AK-47")
	
	camera:init(32, 32, (#map[1] + 1) * 32 - love.graphics.getWidth(), (#map + 1) * 32 - love.graphics.getHeight())
	
	aiCount = 0
	
	respawnPlayer()
end

-- Close
function GameState:close()
end

-- Server Function
local function onReceive(data)
	local Data = loadstring(data)()

	if Data.dataType == "overwritePlayer" then
		Data.dataType = nil
		player.id = Data.id
		
		local i = {}
		sendToServer("requestMap", i)
	end

	if Data.dataType == "die" then
		respawnPlayer()
	end
	
	if Data.dataType == "overwriteMap" then
		Data.dataType = nil
		map = Data

		camera:init(32, 32, (#map[1] + 1) * 32 - love.graphics.getWidth(), (#map + 1) * 32 - love.graphics.getHeight())
		print("Got Map")
		
		local i = {}
		sendToServer("requestMapp", i)
	end
	
	if Data.dataType == "overwriteMapp" then
		Data.dataType = nil
		mapp = Data

		print("Got Mapp")
	end
	
	if Data.dataType == "updatePlayers" then
		Data.dataType = nil
		players = Data
	end
	
	if Data.dataType == "updateEnemies" then
		Data.dataType = nil
		npc = Data
	end

	if Data.dataType == "updateEntities" then
		Data.dataType = nil
		entity.objects = Data
	end

	if Data.dataType == "explosion" then
		Data.dataType = nil
		explosions = Data
	end
end

function sendToServer(dataType, data)
	data.dataType = dataType
	local toSend = table.serialize(data,true,true)
	data.dataType = nil
	
	client:send(toSend)
end
-------------------------------

-- Enable
function GameState:enable()
	print("Initializing Client...")
	love.graphics.setFont(font16)
	
	client = lube.client("tcp")
	client:setHandshake("ProjectCy")
	client:setCallback(onReceive)
	if not client:connect(globalIP, globalPort) then
		enableState("server")
		self:enable()
	else
		print("Initialized Client!")
	
		players = {}
	
		player.ammo = player.ammo + 1
		sendToServer("first", player)
	end
end

-- Disable
function GameState:disable()
	if client then
		client:disconnect()
	end
end

-- Update
function GameState:update(dt)
	client:update(dt)
	camera:updateShake(dt)
	updateNPC(dt)

	player:update(dt)

	--[[if love.keyboard.isDown('q') then
		addNPC("dumbFollow", 250, 250, "Standard", 2, 50)
	end]]--

	player.dataType = "updatePlayer"
	local toSendPlayer = table.serialize(player,true,true)
	client:send(toSendPlayer)
	player.dataType = nil
end

-- Draw
function GameState:draw()
	camera:set()

	-- Rendering Magic
	local tx, ty = camera:relative(0, 0)
	local bx, by = camera:relative(love.graphics.getWidth(), love.graphics.getHeight())
	
	local y_temp =  math.floor(ty / mapp.tileWidth)
	local y2_temp = (by / mapp.tileWidth) + 2
	local x_temp = math.floor(tx / mapp.tileHeight)
	local x2_temp = (bx / mapp.tileHeight) + 2
	
	if y_temp < 1 then
		y_temp = 1
	end
	
	if y2_temp > #map then
		y2_temp = #map
	end
	
	if x_temp < 1 then
		x_temp = 1
	end
	
	if x2_temp > #map[1] then
		x2_temp = #map[1]
	end
	
	for y = y_temp, y2_temp do
		for x = x_temp, x2_temp do
			if map[y][x] ~= -1 then
				love.graphics.draw(tiles[map[y][x]], x * 32, y * 32)
			else
				love.graphics.draw(tiles[0], x * 32, y * 32)
			end
		end
	end
	
	player:draw()
	
	-- Players
	for i,v in ipairs(players) do
		if v.id ~= player.id then
			love.graphics.print(v.nickname, v.x + (v.width / 2) - (font16:getWidth(v.nickname) / 2), v.y - 20)

			local p = 0
			if v.scale == -1 then
				p = 1
			end

			playerAnim[v.action]:draw(playerImg, v.x + (v.width * p), v.y, 0, v.scale, 1)
			
			for k,j in ipairs(v.bullets) do
				love.graphics.draw(guns[j.gun].bullet, j.x, j.y, j.angle + math.rad(90), 1, 1, bullet:getWidth() / 2, bullet:getHeight() / 2)
			end
			
			love.graphics.draw(guns[v.curGun].image, v.x + (v.width / 2), v.y + (v.height / 2) + 5, v.r, v.scale, 1, guns[v.curGun].image:getWidth() / 2, guns[v.curGun].image:getHeight())
			camera:drawMuzzle(v)
		end
	end
	
	drawNPC()
	drawExplosions()

	camera:unset()
	
	love.graphics.setColor(0, 255, 0)
	love.graphics.print(love.timer.getFPS().."\nAI Count: "..aiCount.."\nKill Count: "..player.kills.."\nDeath Count: "..player.deaths.."\nAmmo: "..player.ammo, 0, 0)
	love.graphics.setColor(255, 255, 255)
	
	for i=1,player.health do
		love.graphics.draw(health, love.graphics.getWidth() - i * health:getWidth(), 0)
	end
end

-- MousePressed
function GameState:mousepressed(x, y, button)
	if button == "wu" then
		
	elseif button == "wd" then
		
	end
end

-- KeyPressed
function GameState:keypressed(key, uni)
	if key == "1" then
		equipGun("AK-47")
	elseif key == "2" then
		equipGun("Sniper")
	elseif key == "3" then
		equipGun("Barrel")
	elseif key == "4" then
		equipGun("Bazooka")
	elseif key == "5" then
		equipGun("Tazer")
	end
	
	--[[if key == "q" then
		addNPC("dumbFollow", 250, 250, "Standard", 2, 50)
	end]]--
end

function GameState:quit()
	client:disconnect()
end

function respawnPlayer()
	for y=1, #map do
		for x=1, #map[y] do
			if map[y][x] == -1 then
				player.x = x * 32
				player.y = y * 32
			end
		end
	end

	player.health = player.healthMax
	player.deaths = player.deaths + 1
	player:setInvince(1)
end

function math.clamp(x, min, max)
  return x < min and min or (x > max and max or x)
end