-- Table
ServerState = {}

-- Server Functions

-- All Clients
local function sendToClient(dataType, data)
	data.dataType = dataType
	local toSend = table.serialize(data,true,true)
	data.dataType = nil
	
	server:send(toSend)
end

-- Specified Client
local function sendToSClient(dataType, data, ip)
	data.dataType = dataType
	local toSend = table.serialize(data,true,true)
	data.dataType = nil
	
	server:send(toSend, ip)
end

local function onConnect(ip, port)
	print("Connection from: " .. ip)
end
	
local function onReceive(data, ip, port)
	local Data = loadstring(data)()
	
	if Data.dataType == "first" then
		Data.id = ip
		
		sendToSClient("overwritePlayer", Data, ip)
		
		table.insert(players, Data)
	end
	
	if Data.dataType == "requestMap" then
		Data.dataType = nil
		sendToSClient("overwriteMap", sMap, ip)
	end
	
	if Data.dataType == "requestMapp" then
		Data.dataType = nil
		sendToSClient("overwriteMapp", sMapp, ip)
	end
	
	if Data.dataType == "updatePlayer" then
		Data.dataType = nil
	
		for i,v in ipairs(players) do
			if v.id == Data.id then
				v.x = Data.x
				v.y = Data.y
				v.r = Data.r
				v.scale = Data.scale
				v.action = Data.action
				v.bullets = Data.bullets
				v.curGun = Data.curGun
				v.invincible = Data.invincible
			end
		end
	end
	
	if Data.dataType == "newEnemy" then
		Data.dataType = nil
		Data.id = enemyCount
		enemyCount = enemyCount + 1
		table.insert(enemies, Data)
	end
	
	if Data.dataType == "updateEnemy" then
		Data.dataType = nil
		enemies[Data.number] = Data.table
		
		if enemies[Data.number].health <= 0 then
			table.remove(enemies, Data.number)
		end
	end

	if Data.dataType == "newEntity" then
		Data.dataType = nil
		table.insert(entities, Data)
	end

	if Data.dataType == "removeEntity" then
		Data.dataType = nil
		table.remove(entities, Data.i)
	end

	if Data.dataType == "explode" then
		Data.dataType = nil

		table.insert(booms, Data)
	end
end
	
local function onDisconnect(ip, port)
	for i,v in ipairs(players) do
		if v.id == ip then
			print(v.nickname.." with the IP "..ip.." has Disconnected.")
			table.remove(players, i)
		end
	end
end
-----------------------------------------

-- Enemy Functions
local function checkTile(x, y)
	local mapX, mapY = math.floor(y/mapp.tileWidth), math.floor(x/mapp.tileHeight)
	
	if mapX >= 1 and mapX <= #map and mapY >= 1 and mapY <= #map[1] then 
		if sMap[mapX][mapY] == 1 then
			return false
		else
			return true
		end
	else
		return false
	end
end

local function dist(x1,y1, x2,y2)
	return ((x2-x1)^2+(y2-y1)^2)^0.5
end

local function moveUp(v, dt)
	local canMove = true

	--[[for i,j in ipairs(enemies) do
		if j.id ~= v.id then
			if v.x + v.width >= j.x and v.x <= j.x + j.width and v.y - v.speed * dt <= j.y + j.height then
				canMove = false
				break
			end
		end
	end]]--

	if checkTile(v.x, v.y - v.speed * dt) and checkTile(v.x + v.width, v.y - v.speed * dt) then
		v.y = v.y - v.speed * dt
	end
end

local function moveDown(v, dt)
	local canMove = true

	--[[for i,j in ipairs(enemies) do
		if j.id ~= v.id then
			if v.x + v.width >= j.x and v.x <= j.x + j.width and v.y + v.height + v.speed * dt >= j.y then
				canMove = false
				break
			end
		end
	end]]--
	
	if checkTile(v.x, v.y + v.height + v.speed * dt) and checkTile(v.x + v.width, v.y + v.height + v.speed * dt) then
		v.y = v.y + v.speed * dt
	end
end

local function moveLeft(v, dt)
	local canMove = true

	--[[for i,j in ipairs(enemies) do
		if j.id ~= v.id then
			if v.y + v.height >= j.y and v.y <= j.y + j.height and v.x - v.speed * dt <= j.x + j.width then
				canMove = false
				break
			end
		end
	end]]--

	if checkTile(v.x - v.speed * dt, v.y) and checkTile(v.x - v.speed * dt, v.y + v.height) then
		v.x = v.x - v.speed * dt
	end
end

local function moveRight(v, dt)
	local canMove = true

	--[[for i,j in ipairs(enemies) do
		if j.id ~= v.id then
			if v.y + v.height >= j.y and v.y <= j.y + j.height and v.x + v.width + v.speed * dt >= j.x then
				canMove = false
				break
			end
		end
	end]]--

	if checkTile(v.x + v.width + v.speed * dt, v.y) and checkTile(v.x + v.width + v.speed * dt, v.y + v.height) then
		v.x = v.x + v.speed * dt
	end
end

local function addEnemy(aiType, x, y, type, health, speed)
	local i = {
		ai = aiType,
		type = type,
		width = 24,
		height = 24,
		health = health,
		speed = speed + love.math.random(1, 10),
		x = x,
		y = y,
		alive = true,
		action = "running",
		scale = -1,
	}
	
	table.insert(enemies, i)
end
-----------------------------------

-- New
function ServerState:new()
	local gs = {}

	gs = setmetatable(gs, self)
	self.__index = self
	_gs = gs
	
	return gs
end

-- Load
function ServerState:load()
	globalPort = 10101
	globalIP   = tostring(socket.dns.toip(socket.dns.gethostname()))
end

-- Close
function ServerState:close()
end

-- Enable
function ServerState:enable()
	maxPlayers = 20

	print("Initializing Server...")
	
	-- Starts the Server
	server = lube.server(globalIP, globalPort, "tcp")
	server:setCallback(onReceive, onConnect, onDisconnect)
	server:setHandshake("ProjectCy")
	
	players = {}
	enemies = {}
	entities = {}
	booms = {}
	
	sMap = {
		{ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
		{ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 },
	}
	
	sMapp = {}
	sMapp.tileWidth = 32
	sMapp.tileHeight = 32

	enemyCount = 0
	
	print("Initialized Server!")
end

-- Disable
function ServerState:disable()
end

-- Update
function ServerState:update(dt)
	server:update(dt)
	server:acceptAll()

	-- Players
	sendToClient("updatePlayers", players)
	sendToClient("updateEntities", entities)

	-- Explosions
	for i,v in ipairs(booms) do
		for k,j in ipairs(players) do
			if dist(v.x, v.y, j.x + j.width / 2, j.y + j.height / 2) <= 120 then
				if not j.invincible then
					j.health = j.health - v.size

					if j.health <= 0 then
						local p = {}
						sendToSClient("die", p, j.id)
					end
				end
			end
		end

		for k,j in ipairs(enemies) do
			if dist(v.x, v.y, j.x + j.width / 2, j.y + j.height / 2) <= 120 then
				if j.type ~= "Robot" then
					j.health = j.health - v.size
				end
				
				if j.health <= 0 then
					table.remove(enemies, k)
				end
			end
		end

		v.timer = v.timer - dt

		if v.timer <= 0 then
			table.remove(booms, i)
		end
	end

	sendToClient("explosion", booms)

	-- Zombies
	love.math.setRandomSeed(os.time() * love.math.random(1, 10000000))
		
	for x=1, #sMap[1] do
		for y=1, #sMap do
			if sMap[x][y] == 2 and love.math.random(100) == 1 then
				if #enemies < 50 then
					local j = love.math.random(1, 5)
					
					if j == 1 then
						addNPC("dumbFollow", y * 32, x * 32, "Standard", 2, 50)
					elseif j == 2 then
						addNPC("straightLine", y * 32, x * 32, "Stricker", 10, 100)
					elseif j == 3 then
						addNPC("straightLine", y * 32, x * 32, "Heavy", 1200, 20)
					elseif j == 4 then
						addNPC("straightLine", y * 32, x * 32, "Physic", 1, 40)
					elseif j == 5 then
						addNPC("straightLine", y * 32, x * 32, "Robot", 1, 60)
					end
				end
			end
		end
	end
		
	-- Enemies
	for i,v in ipairs(enemies) do
		local closestPlayer = {
			x = -math.huge,
			y = -math.huge,
			width = 24,
			height = 24,
		}
		
		for k,j in ipairs(players) do
			if dist(closestPlayer.x + closestPlayer.width / 2, closestPlayer.y + closestPlayer.height / 2, v.x + v.width / 2, v.y + v.height) > dist(j.x + j.width / 2, j.y + j.height / 2, v.x + v.width / 2, v.y + v.height / 2) then
				closestPlayer = j
			end
		end
		
		local p = closestPlayer
	
		if v.ai == "dumbFollow" and v.alive then
			if p.x + p.width / 2 > v.x + v.width / 2 then
				v.scale = 1
				moveRight(v, dt)
			end
			
			if p.x + p.width / 2 < v.x + v.width / 2 then
				v.scale = -1
				moveLeft(v, dt)
			end
			
			if p.y + p.height / 2 > v.y + v.height / 2 then
				moveDown(v, dt)
			end
			
			if p.y + p.height / 2 < v.y + v.height / 2 then
				moveUp(v, dt)
			end
		end
		
		if v.ai == "straightLine" and v.alive then
			local x, y = v.x + v.width / 2, v.y + v.height / 2
			local angle = math.atan2((p.y + p.height / 2 - y),(p.x + p.width / 2 - x))
			local dx = 1000 * math.cos(angle)
			local dy = 1000 * math.sin(angle)

			local collided = true

			while not (x >= p.x and x <= p.x + p.width and y >= p.y and y <= p.y + p.height) do
				x = x + (dx * dt)
				y = y + (dy * dt)

				if not checkTile(x, y) then
					collided = false
					break
				end
			end

			if collided then
				v.action = "running"
				if p.x + p.width / 2 > v.x + v.width / 2 then
					v.scale = 1
					moveRight(v, dt)
				end
					
				if p.x + p.width / 2 < v.x + v.width / 2 then
					v.scale = -1
					moveLeft(v, dt)
				end
					
				if p.y + p.height / 2 > v.y + v.height / 2 then
					moveDown(v, dt)
				end
					
				if p.y + p.height / 2 < v.y + v.height / 2 then
					moveUp(v, dt)
				end
			else
				v.action = "idle"
			end
		end
	end
		
	sendToClient("updateEnemies", enemies)
end

-- Draw
function ServerState:draw()
	--love.graphics.printf(table.serialize(players), 0, 200, 800)
end

-- MousePressed
function ServerState:mousepressed(x, y, button)
end