local camera = {
	x = 0,
	y = 0,
	tx = 0,
	ty = 0,
	bx = 0,
	by = 0,
	shake = {},
}

function camera:init(topX, topY, bottomX, bottomY)
	camera.x, camera.y = 0, 0

	camera.tx, camera.ty = topX, topY
	camera.bx, camera.by = bottomX, bottomY

	camera.shake = {}
end

function camera:setPos(x, y)
	if x < camera.tx then x = camera.tx end
	if y < camera.ty then y = camera.ty end
	if x > camera.bx then x = camera.bx end
	if y > camera.by then y = camera.by end
	
	camera.x, camera.y = x, y
end

function camera:set()
	love.graphics.push()

	love.graphics.translate(-camera.x, -camera.y)

	for i,v in ipairs(self.shake) do
		love.graphics.translate(love.math.random(-v.mag, v.mag), love.math.random(-v.mag, v.mag))
	end
end

function camera:unset()
	love.graphics.pop()
end

function camera:relative(x, y)
	return x + camera.x, y + camera.y
end

function camera:startShake(time, magnitude)
	local i = {}
	i.timer = 0
	i.time = time
	i.mag = magnitude
	
	table.insert(self.shake, i)
end

function camera:updateShake(dt)
	for i,v in ipairs(self.shake) do
		v.timer = v.timer + dt
		
		if v.timer >= v.time then
			table.remove(self.shake, i)
		end
	end
end

function camera:drawMuzzle(j)
	for i,v in ipairs(self.shake) do
		love.graphics.draw(muzzle, j.x + (j.width / 2), j.y + (j.height / 2) + 5, j.r, 1, 1, muzzle:getWidth() / 2, guns[j.curGun].image:getHeight() + muzzle:getHeight())
	end
end

return camera