local player = {
	x = 128,
	y = 128,
	width = 24,
	height = 24,
	speed = 150,
	speedMax = 150,
	gunTimer = 0.1,
	gunMax = 0.1,
	ammo = 100,
	damage = 1,
	bulletSpeed = 800,
	curGun = nil,
	health = 5,
	healthMax = 5,
	bullets = {},
	nickname = "Roger",
	r = 0,
	id = 0,
	invince = {},
	invincible = false,
	scale = -1,
	action = "idle",
	kills = 0,
	deaths = -1,
	inv = {},
}

local function checkTile(x, y)
	local mapX, mapY = math.floor(y/mapp.tileWidth), math.floor(x/mapp.tileHeight)
	
	if mapX >= 1 and mapX <= #map and mapY >= 1 and mapY <= #map[1] then 
		if map[mapX][mapY] == 1 then
			return false
		else
			return true
		end
	else
		return false
	end
end

function player:update(dt)
	entity:update(dt)

	playerAnim["idle"]:update(dt)
	playerAnim["running"]:update(dt)

	if love.keyboard.isDown("w") and checkTile(self.x, self.y - self.speed * dt) and checkTile(self.x + self.width, self.y - self.speed * dt) then
		self.y = self.y - self.speed * dt
	end
	
	if love.keyboard.isDown("s") and checkTile(self.x, self.y + self.height + self.speed * dt) and checkTile(self.x + self.width, self.y + self.height + self.speed * dt) then
		self.y = self.y + self.speed * dt
	end
	
	if love.keyboard.isDown("a") and checkTile(self.x - self.speed * dt, self.y) and checkTile(self.x - self.speed * dt, self.y + self.height) then
		self.x = self.x - self.speed * dt
	end
	
	if love.keyboard.isDown("d") and checkTile(self.x + self.width + self.speed * dt, self.y) and checkTile(self.x + self.width + self.speed * dt, self.y + self.height) then
		self.x = self.x + self.speed * dt
	end
	
	if love.keyboard.isDown("w", "s", "a", "d") then
		self.action = "running"
	else
		self.action = "idle"
	end

	self.gunTimer = self.gunTimer + dt

	if love.mouse.isDown("l") then
		self.speed = self.speedMax / 2
		if self.gunTimer >= self.gunMax then
			self:shootGun()
			self.gunTimer = 0
		end
	else
		self.speed = self.speedMax
	end
	
	for i,v in ipairs(self.bullets) do
		v.x = v.x + (v.dx * dt)
		v.y = v.y + (v.dy * dt)
		
		if not checkTile(v.x, v.y) then
			table.remove(self.bullets, i)

			if v.gun == "Bazooka" then
				explode(1200, v.x, v.y)
			end
		end
	end

	local hasInv = false
	for i=1,#self.invince do
		if self.invince[i] then
			if self.invince[i] > 0 then
				hasInv = true
				self.invince[i] = self.invince[i] - dt

				if self.invince[i] <= 0 then
					table.remove(self.invince, i)
				end
			end
		end
	end
	self.invincible = hasInv

	local x, y = camera:relative(love.mouse.getPosition())
	if x > self.x + self.width + love.graphics.getWidth() / 2 - player.width * 10 then
		x = self.x + self.width + love.graphics.getWidth() / 2 - player.width * 10
	end
	if y > self.y + self.height + love.graphics.getHeight() / 2 - player.height * 10 then
		y = self.y + self.height + love.graphics.getHeight() / 2 - player.height * 10
	end

	local topX = self.x + self.width - love.graphics.getWidth() / 2
	local topY = self.y + self.height - love.graphics.getHeight() / 2
	camera:setPos(topX, topY)
end

function player:draw()
	-- self
	love.graphics.setColor(255, 255, 255)

	entity:draw()

	local p = 0
	if self.scale == -1 then
		p = 1
	end

	playerAnim[self.action]:draw(playerImg, self.x + (self.width * p), self.y, 0, self.scale, 1)
	love.graphics.print(self.nickname, self.x + self.width / 2 - (font16:getWidth(self.nickname) / 2), self.y - 20)
	
	-- Bullets
	for i,v in ipairs(self.bullets) do
		love.graphics.draw(guns[v.gun].bullet, v.x, v.y, v.angle + math.rad(90), 1, 1, bullet:getWidth() / 2, bullet:getHeight() / 2)
	end
	
	-- Gun
	local x, y = love.mouse.getPosition()
	x, y = camera:relative(x, y)
	self.r = math.atan2(y - (self.y + (self.height / 2)), x - (self.x + (self.width / 2))) + math.rad(90)

	if math.deg(self.r) > 0 and math.deg(self.r) < 180 then
		self.scale = 1
	else
		self.scale = -1
	end

	love.graphics.draw(guns[self.curGun].image, self.x + (self.width / 2), self.y + (self.height / 2) + 5, self.r, self.scale, 1, guns[self.curGun].image:getWidth() / 2, guns[self.curGun].image:getHeight())
	camera:drawMuzzle(self)
end

function player:shootGun()
	local canShoot = false

	if self.curGun ~= "Tazer" and self.ammo >= 1 then
		canShoot = true
		self.ammo = self.ammo - 1
	end

	if self.curGun == "Tazer" then canShoot = true end

	if canShoot then
		--playSound("gun")
		local x, y = love.mouse.getPosition()
		camera:startShake(guns[self.curGun].timer, guns[self.curGun].speed / 400)

		x, y = camera:relative(x, y)
		
		local startX, startY = self.x + self.width / 2, self.y + self.height / 2 + 5
		local originx, originy = x, y
		
		local fakeAngle = love.math.random(1, 10)
		if fakeAngle > 5 then
			fakeAngle = -(fakeAngle - 10)
		end
		fakeAngle = math.rad(fakeAngle)
		
		local angle = math.atan2((originy - startY),(originx - startX)) + fakeAngle
			
		bulletDx = self.bulletSpeed * math.cos(angle)
		bulletDy = self.bulletSpeed * math.sin(angle)
			
		table.insert(self.bullets, {x = startX, y = startY, dx = bulletDx, dy = bulletDy, angle = angle, gun = self.curGun, damage = self.damage, health = self.damage, type = "enemy"})
	end
end

function player:setInvince(length)
	self.invincible = true
	table.insert(self.invince, length)
end

return player